"""
__init__.py
"""

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n, dialog

import os.path
import Queue
import time

from filter import Filter
from screenbuffer import ScreenBuffer
from tailbook import TailBook
import watchlist

__author__ = "Bas van Oostveen"
__name__ = "MultiTail"
__version__ = "0.3.0"
__license__ = "GPL"

class DefaultConfig:
	""" default config """
	length = 6
	scroll_length = 50
	scroll_jump = 1
	scroll_speed = 0
	enable_intro = True
	enable_logname = False
	enable_direct = False
	enable_linecount = False
	enable_dropshadow = True
	dropshadow_offset = 1
	background_theme = "light"
	font = "Courier New 14"
	crop = 0
	wrap = 0
	wrapat = 0
	watches = []
	_watches = "/var/log/messages"
	colors = []
	_colors = "#000000"
	states = []
	_states = ""

class MultiTail(Sensor):
	def __init__(self, *args):
		""" init """
		global _; _ = i18n.Translator("MultiTail")
		
		# vars
		self.__force_buffer_update = 0
		self.__queue = Queue.Queue(64)
		self.__background_themes = ["transparent","dark","light"]
		self.__background_prefix = "gfx/"
		self.__background_layout = [
			"bg.png",			# left center
			"bg.png",			# top center
			"bg.png",			# right center
			"bg.png",			# bottom center
			"top-left.png",		# top left
			"top-right.png",	# top right
			"bottom-right.png",	# bottom right
			"bottom-left.png"	# bottom left
		]		
		
		# init superclass
		Sensor.__init__(self)
		
		# set config types
		self._set_config_type("length", TYPE_INT, DefaultConfig.length)
		self._set_config_type("scroll_length", TYPE_INT, DefaultConfig.scroll_length)
		self._set_config_type("scroll_jump", TYPE_BOOL, DefaultConfig.scroll_jump)
		self._set_config_type("scroll_speed", TYPE_INT, DefaultConfig.scroll_speed)
		self._set_config_type("enable_intro", TYPE_BOOL, DefaultConfig.enable_intro)
		self._set_config_type("enable_logname", TYPE_BOOL, DefaultConfig.enable_logname)
		self._set_config_type("enable_direct", TYPE_BOOL, DefaultConfig.enable_direct)
		self._set_config_type("enable_dropshadow", TYPE_BOOL, DefaultConfig.enable_dropshadow)
		self._set_config_type("enable_linecount", TYPE_BOOL, DefaultConfig.enable_linecount)
		self._set_config_type("dropshadow_offset", TYPE_INT, DefaultConfig.dropshadow_offset)
		self._set_config_type("background_theme", TYPE_STRING, DefaultConfig.background_theme)
		self._set_config_type("font", TYPE_STRING, DefaultConfig.font)
		self._set_config_type("crop", TYPE_INT, DefaultConfig.crop)
		self._set_config_type("wrap", TYPE_INT, DefaultConfig.wrap)
		self._set_config_type("wrapat", TYPE_INT, DefaultConfig.wrapat)
		self._set_config_type("watches", TYPE_LIST, DefaultConfig._watches)
		self._set_config_type("colors", TYPE_LIST, DefaultConfig._colors)
		self._set_config_type("states", TYPE_LIST, DefaultConfig._states)
		
		# init screenbuffer
		self.__sbuffer = ScreenBuffer(self._get_config)
		
		# init filter
		self.__filter = Filter(self._get_config)
		
		# register
		self._add_timer(0, self.__applet_init)
		self._add_timer(1000, self.__applet_update)
		self._add_thread(self.__command_thread)
		self._watch_config(self.__config_changed)
	
	def __get_theme_layout(self, theme):
		p = self.__background_prefix
		
		if theme in ["trans","transparent"]:
			return p+(",%s"%p).join(["trans.png"]*8)
			
		if theme not in self.__background_themes:
			theme = DefaultConfig.background_theme
		
		p = p + theme+'/'
		l = self.__background_layout
		return p+(',%s'%p).join(l)
	
	def __get_theme_width(self, theme):
		if theme in ["trans","transparent"]:
			return "0,0,0,0"
		else:
			return "5,5,5,5"
	
	def __get_theme_background(self, theme):
		if theme in ["trans","transparent"]:
			return ""
			
		if theme not in self.__background_themes:
			theme = DefaultConfig.background_theme
		return self.__background_prefix+theme+'/bg.png'	
	
	def __setup_logarray(self, output=None):
		length = self._get_config("length")	
		for key in ["font","wrapat","enable_dropshadow","dropshadow_offset"]:
			value = self._get_config(key)
			for c in xrange(length):
				output.set("%s[%d]" % (key, c), value)
		return output
	
	def __setup_background(self, output, theme):
		output.set("frame_uris", self.__get_theme_layout(theme))
		output.set("frame_width", self.__get_theme_width(theme))
		output.set("bg_img", self.__get_theme_background(theme))
		return output
		
	def __config_changed(self, key, value):
		""" config changed event """
		# for these keys:
		
		# update the display
		if key in ["length"]:
			output = self._new_output()
			output.set(key, value)
			self.__setup_logarray(output)
			self._send_output(output)
			self.__force_buffer_update=1
			return
		
		# update the background
		if key in ["background_theme"]:
			output = self._new_output()
			self.__setup_background(output, value)
			self._send_output(output)
			return
		
		# update the logarray
		if key in ["font","wrapat","enable_dropshadow","dropshadow_offset"]:
			output = self._new_output()
			for c in xrange(self._get_config("length")):
				output.set("%s[%d]" % (key, c), value)
			self._send_output(output)
			return
		
		# force buffer update
		if key in ["enable_logname"]:
			self.__force_buffer_update = 1
		
	def __applet_init(self):
		# check theme
		if self._get_config("background_theme") not in self.__background_themes:
			# if theme not matched, fallback to default
			self._set_config("background_theme", DefaultConfig.background_theme)
		
		# show intro
		if self._get_config("enable_intro"):
			self.__sbuffer.showIntro()
		
		# init display
		output = self._new_output()
		
		# setup display
		for key in ["length"]:
			output.set(key, self._get_config(key))
		
		# setup logarray
		self.__setup_logarray(output)
		
		# setup background
		theme = self._get_config("background_theme")
		self.__setup_background(output, theme)
		
		# apply to display
		self._send_output(output)
		
		# never again
		return 0
		
	def __applet_update(self):
		length = self._get_config("length")
		
		output = self._new_output()
		output.set("length", length)

		if self._get_config("enable_linecount"):
			for c in xrange(length):
				output.set("linecount[%d]" % c, c)
		else:
			for c in xrange(length):
				output.set("linecount[%d]" % c, '')
		
		ed = self._get_config("enable_direct")
		ss = self._get_config("scroll_speed")
		qs = self.__queue.qsize()
		if qs==0 and ed==0:
			ss = 0
		elif ed==1:
			ss = qs
		elif ss==0:
			ss = (qs/length) and length-(length/4) or qs
		
		for c in xrange(ss):
			try:
				color, name, line = self.__queue.get(False)
			except Queue.Empty:
				break
				
			for l in self.__filter(line):
				self.__sbuffer.put(color, name, l)
		
		if self.__sbuffer.isChanged() or self.__force_buffer_update:
			self.__force_buffer_update = 0
			c=0
			for color, name, line in self.__sbuffer.list(length):
				if self._get_config("enable_logname"):
					output.set("name[%d]" % c, name)
				else:
					output.set("name[%d]" % c, "")
				output.set("line[%d]" % c, line)
				output.set("linecolor[%d]" % c, color)
				c=c+1
		
		self._send_output(output)
		
		return True
	
	def __command_thread(self):
		tick = 1
		tailbook = TailBook()
		prev_states = []
		prev_watches = []
		prev_colors = []
		while (1):
			if (self._is_stopped()): break
			
			# update tails in tailbook
			new_watches = self._get_config("watches")
			new_colors = self._get_config("colors")
			if (prev_watches!=new_watches) or (prev_colors!=new_colors):
				prev_watches = new_watches
				prev_colors = new_colors
				tailbook.update(new_watches, new_colors)
			
			# check for rotates or resets in files
			tailbook.check()
			
			# set states
			prev_states = self._get_config("states")
			new_states = []
			for tail in new_watches:
				new_states.append(tailbook.getState(tail))
			if prev_states!=new_states:
				self._set_config("states", new_states)
				# on screen messages for changed states
				for i in xrange(len(new_watches)):
					if prev_states[i]!=new_states[i] and not new_states[i]=="ok":
						w = new_watches[i]
						qb = (tailbook.getColor(w), "[%s]" % tailbook.getName(w), new_states[i])
						self.__queue.put(qb)
			
			# get pool
			pool = tailbook.getPool()
			
			# run pool
			while (len(pool)>0):
				for p in pool:
					line = tailbook.getLine(p)
					if line=="":
						del pool[pool.index(p)]
						break # from "for" loop
					
					qb = (tailbook.getColor(p), "[%s]" % tailbook.getName(p), line)
					self.__queue.put(qb)
			
			time.sleep(tick)

	def __open_watch_dialog(self, src, *args):
		if (watchlist.win==None):
			watchlist.show(self._get_config, self._set_config)
		else:
			dialog.warning("Already open", "Watch list already open")
	
	def get_configurator(self):
		""" create configurator """
		c = self._new_configurator()
		c.set_name(_(__name__))
		c.add_title(_("%s version %s" % (__name__,__version__)))
		
		c.add_title(_("Display"))
		c.add_spin(_("Lines on screen:"), "length",
			    _("Number of lines to display on screen"), 1, 127)
		c.add_spin(_("Scroll speed:"), "scroll_speed",
			    _("Number of new lines to scroll per second (0 = automatic)"), 0, 127)
		c.add_checkbox(_("Direct/Fast mode (disables scroll speed)"), "enable_direct",
					   _("Push new lines to the desklet as fast as possible"))
		c.add_checkbox(_("Show intro"), "enable_intro", _("Show Intro"))
		c.add_checkbox(_("Show log filename"), "enable_logname", _("Display the filename before each line"))
		c.add_checkbox(_("Show Linecount"), "enable_linecount", _("Display line numbers"))
		
		c.add_title(_("ScrollBack"))
		c.add_spin(_("Remember # lines:"), "scroll_length",
				_("How many lines to remember for scrollback (0 = disabled)"), 0, 999)
		c.add_checkbox(_("Jump to new line"),"scroll_jump",_("Disable scrolling when new line appears."))
		
		c.add_title(_("Background"))
		c.add_option(_("Background theme"), "background_theme",
			    _("Background Themes"), [[theme]*2 for theme in self.__background_themes])
		
		c.add_title(_("Font"))
		c.add_font_selector(_("Font to use"), "font", _("The font to use."))
		c.add_checkbox(_("Enable dropshadow"), "enable_dropshadow", _("Enable dropshadow"))
		c.add_spin(_("Dropshadow offset"), "dropshadow_offset",
			    _("Offset of the dropshadow text\nfor if you want to make your multitail look fuzzy\nor if you got very strange fonts :)"), 1, 15)
		
		c.add_title(_("Width"))
		c.add_spin(_("Crop:"), "crop",
				   _("Maximum number of chars in one line (0 = disabled"), 0, 256)
		c.add_spin(_("Wrap:"), "wrap",
				   _("Maximum number of chars on one line (0 = disabled"), 0, 256)
		c.add_spin(	_("Label Wrap:"), "wrapat",
					_("Label wrap feature of gdesklets, set in pixels"), 0, 4096)
		
		return c
	
	def call_action(self, action, path, args=[]):
		""" react to user events/actions """
		# open menu
		if (action == "menu"):
			self._open_menu([
				("Watch logs...", 1, None, self.__open_watch_dialog, [])
			])
		elif (action == "scroll"):
			direction = args[0]
			if direction==0:
				self.__sbuffer.scroll_up()
			elif direction==1:
				self.__sbuffer.scroll_down()
			self.__applet_update()
	
	def _shutdown(self):
		pass

def new_sensor(args): return apply(MultiTail, args)
